@@ -1,52 +1,50 @@
-# -*- coding: utf-8 -*-
 """Aliases for the xonsh shell."""
+import argparse
+import collections.abc as cabc
+import functools
+import inspect
 import os
 import re
 import sys
-import inspect
-import argparse
-import collections.abc as cabc
-
-from xonsh.built_ins import XSH
-from xonsh.lazyasd import lazyobject
-from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
-from xonsh.environ import locate_binary, make_args_env
-from xonsh.foreign_shells import foreign_shell_data
-from xonsh.jobs import jobs, fg, bg, clean_jobs, disown
-from xonsh.platform import (
-    ON_ANACONDA,
-    ON_DARWIN,
-    ON_WINDOWS,
-    ON_FREEBSD,
-    ON_NETBSD,
-    ON_OPENBSD,
-    ON_DRAGONFLY,
-    ON_POSIX,
-    IN_APPIMAGE,
-)
-from xonsh.tools import (
-    XonshError,
-    argvquote,
-    escape_windows_cmd_string,
-    to_bool,
-    swap_values,
-    strip_simple_quotes,
-    ALIAS_KWARG_NAMES,
-    unthreadable,
-    print_color,
-    to_repr_pretty_,
-)
-from xonsh.timings import timeit_alias
-from xonsh.xontribs import xontribs_main
-from xonsh.ast import isexpression
+import types
+import typing as tp
 
 import xonsh.completers._aliases as xca
 import xonsh.history.main as xhm
 import xonsh.xoreutils.which as xxw
-from xonsh.xoreutils.umask import umask
-
-if ON_POSIX:
-    from xonsh.xoreutils.ulimit import ulimit
+from xonsh.ast import isexpression
+from xonsh.built_ins import XSH
+from xonsh.cli_utils import Annotated, Arg, ArgParserAlias
+from xonsh.dirstack import _get_cwd, cd, dirs, popd, pushd
+from xonsh.environ import locate_binary, make_args_env
+from xonsh.foreign_shells import foreign_shell_data
+from xonsh.jobs import bg, clean_jobs, disown, fg, jobs
+from xonsh.lazyasd import lazyobject
+from xonsh.platform import (
+    IN_APPIMAGE,
+    ON_ANACONDA,
+    ON_DARWIN,
+    ON_DRAGONFLY,
+    ON_FREEBSD,
+    ON_NETBSD,
+    ON_OPENBSD,
+    ON_WINDOWS,
+)
+from xonsh.timings import timeit_alias
+from xonsh.tools import (
+    ALIAS_KWARG_NAMES,
+    XonshError,
+    adjust_shlvl,
+    argvquote,
+    escape_windows_cmd_string,
+    print_color,
+    strip_simple_quotes,
+    swap_values,
+    to_repr_pretty_,
+    to_shlvl,
+    unthreadable,
+)
+from xonsh.xontribs import xontribs_main
 
 
 @lazyobject
@@ -61,6 +59,45 @@ class Aliases(cabc.MutableMapping):
         self._raw = {}
         self.update(*args, **kwargs)
 
+    @staticmethod
+    def _get_func_name(func):
+        name = func.__name__
+
+        # Strip leading underscore
+        if name.startswith("_"):
+            name = name[1:]
+        return name
+
+    def _register(self, func, name="", dash_case=True):
+        name = name or self._get_func_name(func)
+
+        if dash_case:
+            name = name.replace("_", "-")
+
+        self[name] = func
+        return func
+
+    @tp.overload
+    def register(self, func: types.FunctionType) -> types.FunctionType:
+        """simple usage"""
+
+    @tp.overload
+    def register(
+        self, name: str, *, dash_case: bool = True
+    ) -> tp.Callable[[types.FunctionType], types.FunctionType]:
+        ...
+
+    def register(self, func_or_name, name=None, dash_case=True):
+        """Decorator to register the given function by name."""
+
+        if isinstance(func_or_name, types.FunctionType):
+            return self._register(func_or_name, name, dash_case)
+
+        def wrapper(func):
+            return self._register(func, func_or_name, dash_case)
+
+        return wrapper
+
     def get(self, key, default=None):
         """Returns the (possibly modified) value. If the key is not present,
         then `default` is returned.
@@ -181,7 +218,7 @@ class Aliases(cabc.MutableMapping):
         return str(self._raw)
 
     def __repr__(self):
-        return "{0}.{1}({2})".format(
+        return "{}.{}({})".format(
             self.__class__.__module__, self.__class__.__name__, self._raw
         )
 
@@ -218,9 +255,11 @@ class ExecAlias:
                 locs=frame.f_locals,
                 filename=self.filename,
             )
+        if XSH.history is not None:
+            return XSH.history.last_cmd_rtn
 
     def __repr__(self):
-        return "ExecAlias({0!r}, filename={1!r})".format(self.src, self.filename)
+        return f"ExecAlias({self.src!r}, filename={self.filename!r})"
 
 
 class PartialEvalAliasBase:
@@ -367,178 +406,123 @@ def xonsh_reset(args, stdin=None):
     XSH.ctx.clear()
 
 
-@lazyobject
-def _SOURCE_FOREIGN_PARSER():
-    desc = "Sources a file written in a foreign shell language."
-    parser = argparse.ArgumentParser("source-foreign", description=desc)
-    parser.add_argument("shell", help="Name or path to the foreign shell")
-    parser.add_argument(
-        "files_or_code",
-        nargs="+",
-        help="file paths to source or code in the target " "language.",
-    )
-    parser.add_argument(
-        "-i",
-        "--interactive",
-        type=to_bool,
-        default=True,
-        help="whether the sourced shell should be interactive",
-        dest="interactive",
-    )
-    parser.add_argument(
-        "-l",
-        "--login",
-        type=to_bool,
-        default=False,
-        help="whether the sourced shell should be login",
-        dest="login",
-    )
-    parser.add_argument(
-        "--envcmd", default=None, dest="envcmd", help="command to print environment"
-    )
-    parser.add_argument(
-        "--aliascmd", default=None, dest="aliascmd", help="command to print aliases"
-    )
-    parser.add_argument(
-        "--extra-args",
-        default=(),
-        dest="extra_args",
-        type=(lambda s: tuple(s.split())),
-        help="extra arguments needed to run the shell",
-    )
-    parser.add_argument(
-        "-s",
-        "--safe",
-        type=to_bool,
-        default=True,
-        help="whether the source shell should be run safely, "
-        "and not raise any errors, even if they occur.",
-        dest="safe",
-    )
-    parser.add_argument(
-        "-p",
-        "--prevcmd",
-        default=None,
-        dest="prevcmd",
-        help="command(s) to run before any other commands, "
-        "replaces traditional source.",
-    )
-    parser.add_argument(
-        "--postcmd",
-        default="",
-        dest="postcmd",
-        help="command(s) to run after all other commands",
-    )
-    parser.add_argument(
-        "--funcscmd",
-        default=None,
-        dest="funcscmd",
-        help="code to find locations of all native functions " "in the shell language.",
-    )
-    parser.add_argument(
-        "--sourcer",
-        default=None,
-        dest="sourcer",
-        help="the source command in the target shell " "language, default: source.",
-    )
-    parser.add_argument(
-        "--use-tmpfile",
-        type=to_bool,
-        default=False,
-        help="whether the commands for source shell should be "
-        "written to a temporary file.",
-        dest="use_tmpfile",
-    )
-    parser.add_argument(
-        "--seterrprevcmd",
-        default=None,
-        dest="seterrprevcmd",
-        help="command(s) to set exit-on-error before any" "other commands.",
-    )
-    parser.add_argument(
-        "--seterrpostcmd",
-        default=None,
-        dest="seterrpostcmd",
-        help="command(s) to set exit-on-error after all" "other commands.",
-    )
-    parser.add_argument(
-        "--overwrite-aliases",
-        default=False,
-        action="store_true",
-        dest="overwrite_aliases",
-        help="flag for whether or not sourced aliases should "
-        "replace the current xonsh aliases.",
-    )
-    parser.add_argument(
-        "--suppress-skip-message",
-        default=None,
-        action="store_true",
-        dest="suppress_skip_message",
-        help="flag for whether or not skip messages should be suppressed.",
-    )
-    parser.add_argument(
-        "--show",
-        default=False,
-        action="store_true",
-        dest="show",
-        help="Will show the script output.",
-    )
-    parser.add_argument(
-        "-d",
-        "--dry-run",
-        default=False,
-        action="store_true",
-        dest="dryrun",
-        help="Will not actually source the file.",
-    )
-    return parser
+def source_foreign_fn(
+    shell: str,
+    files_or_code: Annotated[tp.List[str], Arg(nargs="+")],
+    interactive=True,
+    login=False,
+    envcmd=None,
+    aliascmd=None,
+    extra_args="",
+    safe=True,
+    prevcmd="",
+    postcmd="",
+    funcscmd="",
+    sourcer=None,
+    use_tmpfile=False,
+    seterrprevcmd=None,
+    seterrpostcmd=None,
+    overwrite_aliases=False,
+    suppress_skip_message=False,
+    show=False,
+    dryrun=False,
+    _stderr=None,
+):
+    """Sources a file written in a foreign shell language.
 
-
-def source_foreign(args, stdin=None, stdout=None, stderr=None):
-    """Sources a file written in a foreign shell language."""
+    Parameters
+    ----------
+    shell
+        Name or path to the foreign shell
+    files_or_code
+        file paths to source or code in the target language.
+    interactive : -n, --non-interactive
+        whether the sourced shell should be interactive
+    login : -l, --login
+        whether the sourced shell should be login
+    envcmd : --envcmd
+        command to print environment
+    aliascmd : --aliascmd
+        command to print aliases
+    extra_args : --extra-args
+        extra arguments needed to run the shell
+    safe : -u, --unsafe
+        whether the source shell should be run safely, and not raise any errors, even if they occur.
+    prevcmd : -p, --prevcmd
+        command(s) to run before any other commands, replaces traditional source.
+    postcmd : --postcmd
+        command(s) to run after all other commands
+    funcscmd : --funcscmd
+        code to find locations of all native functions in the shell language.
+    sourcer : --sourcer
+        the source command in the target shell language.
+        If this is not set, a default value will attempt to be
+        looked up based on the shell name.
+    use_tmpfile : --use-tmpfile
+        whether the commands for source shell should be written to a temporary file.
+    seterrprevcmd : --seterrprevcmd
+        command(s) to set exit-on-error before any other commands.
+    seterrpostcmd : --seterrpostcmd
+        command(s) to set exit-on-error after all other commands.
+    overwrite_aliases : --overwrite-aliases
+        flag for whether or not sourced aliases should replace the current xonsh aliases.
+    suppress_skip_message : --suppress-skip-message
+        flag for whether or not skip messages should be suppressed.
+    show : --show
+        show the script output.
+    dryrun : -d, --dry-run
+        Will not actually source the file.
+    """
+    extra_args = tuple(extra_args.split())
     env = XSH.env
-    ns = _SOURCE_FOREIGN_PARSER.parse_args(args)
-    ns.suppress_skip_message = (
+    suppress_skip_message = (
         env.get("FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE")
-        if ns.suppress_skip_message is None
-        else ns.suppress_skip_message
+        if not suppress_skip_message
+        else suppress_skip_message
     )
-    if ns.prevcmd is not None:
+    files: tp.Tuple[str, ...] = ()
+    if prevcmd:
         pass  # don't change prevcmd if given explicitly
-    elif os.path.isfile(ns.files_or_code[0]):
-        # we have filename to source
-        ns.prevcmd = '{} "{}"'.format(ns.sourcer, '" "'.join(ns.files_or_code))
-    elif ns.prevcmd is None:
-        ns.prevcmd = " ".join(ns.files_or_code)  # code to run, no files
+    elif os.path.isfile(files_or_code[0]):
+        if not sourcer:
+            return (None, "xonsh: error: `sourcer` command is not mentioned.\n", 1)
+        # we have filenames to source
+        prevcmd = "".join([f"{sourcer} {f}\n" for f in files_or_code])
+        files = tuple(files_or_code)
+    elif not prevcmd:
+        prevcmd = " ".join(files_or_code)  # code to run, no files
     foreign_shell_data.cache_clear()  # make sure that we don't get prev src
     fsenv, fsaliases = foreign_shell_data(
-        shell=ns.shell,
-        login=ns.login,
-        interactive=ns.interactive,
-        envcmd=ns.envcmd,
-        aliascmd=ns.aliascmd,
-        extra_args=ns.extra_args,
-        safe=ns.safe,
-        prevcmd=ns.prevcmd,
-        postcmd=ns.postcmd,
-        funcscmd=ns.funcscmd,
-        sourcer=ns.sourcer,
-        use_tmpfile=ns.use_tmpfile,
-        seterrprevcmd=ns.seterrprevcmd,
-        seterrpostcmd=ns.seterrpostcmd,
-        show=ns.show,
-        dryrun=ns.dryrun,
+        shell=shell,
+        login=login,
+        interactive=interactive,
+        envcmd=envcmd,
+        aliascmd=aliascmd,
+        extra_args=extra_args,
+        safe=safe,
+        prevcmd=prevcmd,
+        postcmd=postcmd,
+        funcscmd=funcscmd or None,  # the default is None in the called function
+        sourcer=sourcer,
+        use_tmpfile=use_tmpfile,
+        seterrprevcmd=seterrprevcmd,
+        seterrpostcmd=seterrpostcmd,
+        show=show,
+        dryrun=dryrun,
+        files=files,
     )
     if fsenv is None:
-        if ns.dryrun:
+        if dryrun:
             return
         else:
-            msg = "xonsh: error: Source failed: {0!r}\n".format(ns.prevcmd)
+            msg = f"xonsh: error: Source failed: {prevcmd!r}\n"
             msg += "xonsh: error: Possible reasons: File not found or syntax error\n"
             return (None, msg, 1)
     # apply results
     denv = env.detype()
     for k, v in fsenv.items():
+        if k == "SHLVL":  # ignore $SHLVL as sourcing should not change $SHLVL
+            continue
         if k in denv and v == denv[k]:
             continue  # no change from original
         env[k] = v
@@ -551,9 +535,9 @@ def source_foreign(args, stdin=None, stdout=None, stderr=None):
     for k, v in fsaliases.items():
         if k in baliases and v == baliases[k]:
             continue  # no change from original
-        elif ns.overwrite_aliases or k not in baliases:
+        elif overwrite_aliases or k not in baliases:
             baliases[k] = v
-        elif ns.suppress_skip_message:
+        elif suppress_skip_message:
             pass
         else:
             msg = (
@@ -563,7 +547,12 @@ def source_foreign(args, stdin=None, stdout=None, stderr=None):
                 'You may prevent this message with "--suppress-skip-message" or '
                 '"$FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE = True".'
             )
-            print(msg.format(k, ns.shell), file=stderr)
+            print(msg.format(k, shell), file=_stderr)
+
+
+source_foreign = ArgParserAlias(
+    func=source_foreign_fn, has_args=True, prog="source-foreign"
+)
 
 
 @unthreadable
@@ -581,7 +570,7 @@ def source_alias(args, stdin=None):
             fpath = locate_binary(fname)
             if fpath is None:
                 if env.get("XONSH_DEBUG"):
-                    print("source: {}: No such file".format(fname), file=sys.stderr)
+                    print(f"source: {fname}: No such file", file=sys.stderr)
                 if i == 0:
                     raise RuntimeError(
                         "must source at least one file, " + fname + " does not exist."
@@ -595,7 +584,7 @@ def source_alias(args, stdin=None):
                 "then please use the appropriate source command. "
                 "For example, source-bash script.sh"
             )
-        with open(fpath, "r", encoding=encoding, errors=errors) as fp:
+        with open(fpath, encoding=encoding, errors=errors) as fp:
             src = fp.read()
         if not src.endswith("\n"):
             src += "\n"
@@ -616,44 +605,117 @@ def source_alias(args, stdin=None):
                 raise
 
 
-def source_cmd(args, stdin=None):
-    """Simple cmd.exe-specific wrapper around source-foreign."""
-    args = list(args)
+def source_cmd_fn(
+    files: Annotated[tp.List[str], Arg(nargs="+")],
+    login=False,
+    aliascmd=None,
+    extra_args="",
+    safe=True,
+    postcmd="",
+    funcscmd="",
+    seterrprevcmd=None,
+    overwrite_aliases=False,
+    suppress_skip_message=False,
+    show=False,
+    dryrun=False,
+    _stderr=None,
+):
+    """
+        Source cmd.exe files
+
+    Parameters
+    ----------
+    files
+        paths to source files.
+    login : -l, --login
+        whether the sourced shell should be login
+    envcmd : --envcmd
+        command to print environment
+    aliascmd : --aliascmd
+        command to print aliases
+    extra_args : --extra-args
+        extra arguments needed to run the shell
+    safe : -s, --safe
+        whether the source shell should be run safely, and not raise any errors, even if they occur.
+    postcmd : --postcmd
+        command(s) to run after all other commands
+    funcscmd : --funcscmd
+        code to find locations of all native functions in the shell language.
+    seterrprevcmd : --seterrprevcmd
+        command(s) to set exit-on-error before any other commands.
+    overwrite_aliases : --overwrite-aliases
+        flag for whether or not sourced aliases should replace the current xonsh aliases.
+    suppress_skip_message : --suppress-skip-message
+        flag for whether or not skip messages should be suppressed.
+    show : --show
+        show the script output.
+    dryrun : -d, --dry-run
+        Will not actually source the file.
+    """
+    args = list(files)
     fpath = locate_binary(args[0])
     args[0] = fpath if fpath else args[0]
     if not os.path.isfile(args[0]):
-        return (None, "xonsh: error: File not found: {}\n".format(args[0]), 1)
+        return (None, f"xonsh: error: File not found: {args[0]}\n", 1)
     prevcmd = "call "
     prevcmd += " ".join([argvquote(arg, force=True) for arg in args])
     prevcmd = escape_windows_cmd_string(prevcmd)
-    args.append("--prevcmd={}".format(prevcmd))
-    args.insert(0, "cmd")
-    args.append("--interactive=0")
-    args.append("--sourcer=call")
-    args.append("--envcmd=set")
-    args.append("--seterrpostcmd=if errorlevel 1 exit 1")
-    args.append("--use-tmpfile=1")
     with XSH.env.swap(PROMPT="$P$G"):
-        return source_foreign(args, stdin=stdin)
+        return source_foreign_fn(
+            shell="cmd",
+            files_or_code=args,
+            interactive=True,
+            sourcer="call",
+            envcmd="set",
+            seterrpostcmd="if errorlevel 1 exit 1",
+            use_tmpfile=True,
+            prevcmd=prevcmd,
+            #     from this function
+            login=login,
+            aliascmd=aliascmd,
+            extra_args=extra_args,
+            safe=safe,
+            postcmd=postcmd,
+            funcscmd=funcscmd,
+            seterrprevcmd=seterrprevcmd,
+            overwrite_aliases=overwrite_aliases,
+            suppress_skip_message=suppress_skip_message,
+            show=show,
+            dryrun=dryrun,
+        )
 
 
-def xexec(args, stdin=None):
-    """exec [-h|--help] [-cl] [-a name] command [args...]
+source_cmd = ArgParserAlias(func=source_cmd_fn, has_args=True, prog="source-cmd")
 
-    exec (also aliased as xexec) uses the os.execvpe() function to
-    replace the xonsh process with the specified program. This provides
-    the functionality of the bash 'exec' builtin::
+
+def xexec_fn(
+    command: Annotated[tp.List[str], Arg(nargs=argparse.REMAINDER)],
+    login=False,
+    clean=False,
+    name="",
+    _stdin=None,
+):
+    """exec (also aliased as xexec) uses the os.execvpe() function to
+    replace the xonsh process with the specified program.
+
+    This provides the functionality of the bash 'exec' builtin::
 
         >>> exec bash -l -i
         bash $
 
-    The '-h' and '--help' options print this message and exit.
-    If the '-l' option is supplied, the shell places a dash at the
-    beginning of the zeroth argument passed to command to simulate login
-    shell.
-    The '-c' option causes command to be executed with an empty environment.
-    If '-a' is supplied, the shell passes name as the zeroth argument
-    to the executed command.
+    Parameters
+    ----------
+    command
+        program to launch along its arguments
+    login : -l, --login
+        the shell places a dash at the
+        beginning of the zeroth argument passed to command to simulate login
+        shell.
+    clean : -c, --clean
+        causes command to be executed with an empty environment.
+    name : -a, --name
+        the shell passes name as the zeroth argument
+        to the executed command.
 
     Notes
     -----
@@ -665,66 +727,43 @@ def xexec(args, stdin=None):
     explicitly with ![exec command]. For more details, please see
     http://xon.sh/faq.html#exec.
     """
-    if len(args) == 0:
-        return (None, "xonsh: exec: no args specified\n", 1)
-
-    parser = argparse.ArgumentParser(add_help=False)
-    parser.add_argument("-h", "--help", action="store_true")
-    parser.add_argument("-l", dest="login", action="store_true")
-    parser.add_argument("-c", dest="clean", action="store_true")
-    parser.add_argument("-a", dest="name", nargs="?")
-    parser.add_argument("command", nargs=argparse.REMAINDER)
-    args = parser.parse_args(args)
-
-    if args.help:
-        return inspect.getdoc(xexec)
-
-    if len(args.command) == 0:
+    if len(command) == 0:
         return (None, "xonsh: exec: no command specified\n", 1)
 
-    command = args.command[0]
-    if args.name is not None:
-        args.command[0] = args.name
-    if args.login:
-        args.command[0] = "-{}".format(args.command[0])
+    cmd = command[0]
+    if name:
+        command[0] = name
+    if login:
+        command[0] = f"-{command[0]}"
 
     denv = {}
-    if not args.clean:
+    if not clean:
         denv = XSH.env.detype()
 
+        # decrement $SHLVL to mirror bash's behaviour
+        if "SHLVL" in denv:
+            old_shlvl = to_shlvl(denv["SHLVL"])
+            denv["SHLVL"] = str(adjust_shlvl(old_shlvl, -1))
+
     try:
-        os.execvpe(command, args.command, denv)
+        os.execvpe(cmd, command, denv)
     except FileNotFoundError as e:
         return (
             None,
-            "xonsh: exec: file not found: {}: {}"
-            "\n".format(e.args[1], args.command[0]),
+            "xonsh: exec: file not found: {}: {}" "\n".format(e.args[1], command[0]),
             1,
         )
 
 
-class AWitchAWitch(argparse.Action):
-    SUPPRESS = "==SUPPRESS=="
-
-    def __init__(
-        self, option_strings, version=None, dest=SUPPRESS, default=SUPPRESS, **kwargs
-    ):
-        super().__init__(
-            option_strings=option_strings, dest=dest, default=default, nargs=0, **kwargs
-        )
-
-    def __call__(self, parser, namespace, values, option_string=None):
-        import webbrowser
-
-        webbrowser.open("https://github.com/xonsh/xonsh/commit/f49b400")
-        parser.exit()
+xexec = ArgParserAlias(func=xexec_fn, has_args=True, prog="xexec")
 
 
-def xonfig(args, stdin=None):
+@lazyobject
+def xonfig():
     """Runs the xonsh configuration utility."""
     from xonsh.xonfig import xonfig_main  # lazy import
 
-    return xonfig_main(args)
+    return xonfig_main
 
 
 @unthreadable
@@ -776,7 +815,7 @@ def detect_xpip_alias():
             # XXX: Does windows have an installation mode that requires UAC?
             return basecmd
         elif not os.access(os.path.dirname(sys.executable), os.W_OK):
-            return ["sudo"] + basecmd
+            return basecmd.extend(["--user"])
         else:
             return basecmd
     except Exception:
@@ -801,14 +840,21 @@ def make_default_aliases():
         "exec": xexec,
         "xexec": xexec,
         "source": source_alias,
-        "source-zsh": ["source-foreign", "zsh", "--sourcer=source"],
-        "source-bash": ["source-foreign", "bash", "--sourcer=source"],
+        "source-zsh": ArgParserAlias(
+            func=functools.partial(source_foreign_fn, "zsh", sourcer="source"),
+            has_args=True,
+            prog="source-zsh",
+        ),
+        "source-bash": ArgParserAlias(
+            func=functools.partial(source_foreign_fn, "bash", sourcer="source"),
+            has_args=True,
+            prog="source-bash",
+        ),
         "source-cmd": source_cmd,
         "source-foreign": source_foreign,
         "history": xhm.history_main,
         "trace": trace,
         "timeit": timeit_alias,
-        "umask": umask,
         "xonfig": xonfig,
         "scp-resume": ["rsync", "--partial", "-h", "--progress", "--rsh=ssh"],
         "showcmd": showcmd,
@@ -819,8 +865,6 @@ def make_default_aliases():
         "xpip": detect_xpip_alias(),
         "xonsh-reset": xonsh_reset,
     }
-    if ON_POSIX:
-        default_aliases["ulimit"] = ulimit
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
         windows_cmd_aliases = {
